#! /usr/bin/python -u
# Copyright 2012 Cumulus Networks LLC, all rights reserved

# This program is a Net-SNMP pass persist script that services the
# Resource Query MIB that's been defined by Cumulus Networks.

import subprocess
import syslog
import snmp_passpersist as snmp

oid_base = '.1.3.6.1.4.1.40310.2'

# change the debug to test without snmpd
debug = 0
already_logged = False

def pp_add_cnt(oid, val):
    if debug:
        print '%s : %s' %(oid, val)
    else:
        pp.add_cnt_32bit(oid, val)

def pp_add_str(oid, val):
    if debug:
        print '%s : %s' %(oid, val)
    else:
        pp.add_str(oid, val)

def update_discard_cntrs(port_list):
    """
    Reads latest values for counters from the system using
    ethtool and updates the cached values.
    """
    # update the interface names with counters
    # port_data is an array indexed by the port index given by ip link show
    # it contains port_data[indexCounter]['portName'] and
    # port_data[indexCounter][ethtool counter name]
    global already_logged
    port_data = dict()
    for port in port_list:
        oline = port.split(':')
        indexCounter = int(oline[0].strip())
        portName = oline[1].strip()
        # Only handle hardware swp ports
        if "swp" not in portName or "@" in portName or "." in portName:
            continue

        try:
            ethtool_lines = subprocess.check_output((['/sbin/ethtool','-S', portName]),
                                                    stderr=subprocess.STDOUT,
                                                    shell=False).splitlines()
        except BaseException as e:
            if not already_logged:
                syslog.syslog('cl_drop_cntrs_pp: ethtool EXCEPTION=%s' % e)
                already_logged = True
            ethtool_lines = []


        stats_dict = {}
        # port_data is an array indexed by the ip link show index value
        # we will print this out in order later
        for l in ethtool_lines:
            (k, v) = l.split(':')
            stats_dict[k.strip()] = v.strip()
        port_data[indexCounter] = {}
        port_data[indexCounter]['portName'] = portName
        drop_counters = ['HwIfInL3Drops','HwIfInBufferDrops','HwIfInAclDrops',
                         'HwIfOutQDrops','HwIfOutNonQDrops']
        interface_counters = ['HwIfInOctets', 'HwIfInUcastPkts',
                              'HwIfInBcastPkts', 'HwIfInMcastPkts',
                              'HwIfOutOctets','HwIfOutUcastPkts',
                              'HwIfOutBcastPkts','HwIfOutMcastPkts']
        all_counters = drop_counters + interface_counters
        # save only what we need
        for keepItem in all_counters:
            # if it does not exist, just default to 0
            port_data[indexCounter][keepItem] = stats_dict.get(keepItem,0)

    try:
        # loop over the sysSpecificCounters.discardCounters discards table
        # then the then the sysSpecificCounters.interfaceCounters counters table
        for baseoid,keeper in [('1.1.1',drop_counters),
                               ('2.1.1',interface_counters)]:
            # Print the port names first
            portindex = port_data.keys()
            portindex.sort()
            for pi in portindex:
                pp_add_str("%s.%d.%d"%(baseoid, 1, pi),
                           port_data[pi]['portName'])
            # for each counter, loop over all interfaces and print it
            for keepItem in keeper:
                countIndex = keeper.index(keepItem) + 2
                for pi in portindex:
                    pp_add_cnt("%s.%d.%d"%(baseoid, countIndex, pi),
                               int(port_data[pi][keepItem]))

    except Exception as e:
        syslog.syslog('cl_drop_cntrs_pp.py: Exception=%s' % e)


def update():
    # Build list of ports and ifIndices first
    port_list = subprocess.check_output((['/sbin/ip','-o','link', 'show']),
                                        stderr=subprocess.STDOUT,
                                        shell=False).splitlines()
    # update the interface names with counters
    update_discard_cntrs(port_list)


if debug:
    update()
else:
    pp = snmp.PassPersist(oid_base)
    pp.start(update, 30)
